@@ -1,8 +1,15 @@
+#[cfg(feature = "async")]
+pub mod async_read_write;
+#[cfg(feature = "async")]
+mod async_traits;
+
 mod error;
 mod int;
+mod shape;
 mod traits;
 
 pub use crate::error::Error;
+pub use crate::shape::Shape;
 pub use crate::traits::{BinProtRead, BinProtSize, BinProtWrite};
 
 use byteorder::{LittleEndian, ReadBytesExt, WriteBytesExt};
@@ -27,6 +34,27 @@ pub fn binprot_read_with_size<R: Read, B: BinProtRead>(r: &mut R) -> Result<B, E
 #[derive(Debug, Clone, PartialEq, Eq, Copy)]
 pub struct Nat0(pub u64);
 
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub struct Bytes(pub Vec<u8>);
+
+impl std::convert::From<String> for Bytes {
+    fn from(str: String) -> Self {
+        Bytes(str.into_bytes())
+    }
+}
+
+impl std::convert::From<&str> for Bytes {
+    fn from(str: &str) -> Self {
+        Bytes(str.as_bytes().to_vec())
+    }
+}
+
+impl std::convert::From<Vec<u8>> for Bytes {
+    fn from(v: Vec<u8>) -> Self {
+        Bytes(v)
+    }
+}
+
 impl BinProtWrite for Nat0 {
     fn binprot_write<W: Write>(&self, w: &mut W) -> std::io::Result<()> {
         int::write_nat0(w, self.0)
@@ -70,6 +98,12 @@ impl<T: BinProtWrite> BinProtWrite for Option<T> {
     }
 }
 
+impl<T: BinProtWrite> BinProtWrite for Box<T> {
+    fn binprot_write<W: Write>(&self, w: &mut W) -> std::io::Result<()> {
+        self.as_ref().binprot_write(w)
+    }
+}
+
 impl<T: BinProtWrite> BinProtWrite for Vec<T> {
     fn binprot_write<W: Write>(&self, w: &mut W) -> std::io::Result<()> {
         int::write_nat0(w, self.len() as u64)?;
@@ -94,7 +128,7 @@ impl BinProtWrite for String {
     fn binprot_write<W: Write>(&self, w: &mut W) -> std::io::Result<()> {
         let bytes = self.as_bytes();
         int::write_nat0(w, bytes.len() as u64)?;
-        w.write_all(&bytes)
+        w.write_all(bytes)
     }
 }
 
@@ -102,7 +136,15 @@ impl BinProtWrite for &str {
     fn binprot_write<W: Write>(&self, w: &mut W) -> std::io::Result<()> {
         let bytes = self.as_bytes();
         int::write_nat0(w, bytes.len() as u64)?;
-        w.write_all(&bytes)
+        w.write_all(bytes)
+    }
+}
+
+impl BinProtWrite for Bytes {
+    fn binprot_write<W: Write>(&self, w: &mut W) -> std::io::Result<()> {
+        let bytes = &self.0;
+        int::write_nat0(w, bytes.len() as u64)?;
+        w.write_all(bytes)
     }
 }
 
@@ -243,6 +285,16 @@ impl<T: BinProtRead> BinProtRead for Option<T> {
     }
 }
 
+impl<T: BinProtRead> BinProtRead for Box<T> {
+    fn binprot_read<R: Read + ?Sized>(r: &mut R) -> Result<Self, Error>
+    where
+        Self: Sized,
+    {
+        let v = T::binprot_read(r)?;
+        Ok(Box::new(v))
+    }
+}
+
 impl<T: BinProtRead> BinProtRead for Vec<T> {
     fn binprot_read<R: Read + ?Sized>(r: &mut R) -> Result<Self, Error>
     where
@@ -307,6 +359,20 @@ impl BinProtRead for String {
     }
 }
 
+impl BinProtRead for Bytes {
+    fn binprot_read<R: Read + ?Sized>(r: &mut R) -> Result<Self, Error>
+    where
+        Self: Sized,
+    {
+        let len = int::read_nat0(r)?;
+        let mut buf: Vec<u8> = vec![0u8; len as usize];
+        r.read_exact(&mut buf)?;
+        Ok(Bytes(buf))
+    }
+}
+
+/// A value serialized by first having its size as a nat0, then the
+/// encoding of the value itself.
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct WithLen<T>(pub T);
 
@@ -329,3 +395,28 @@ impl<T: BinProtRead> BinProtRead for WithLen<T> {
         Ok(WithLen(t))
     }
 }
+
+/// A buffer serialized as its size first as a nat0, then the payload itself.
+#[derive(Debug, Clone, PartialEq)]
+pub struct BufferWithLen(pub Vec<u8>);
+
+impl BinProtRead for BufferWithLen {
+    fn binprot_read<R: std::io::Read + ?Sized>(r: &mut R) -> Result<Self, Error>
+    where
+        Self: Sized,
+    {
+        let len = Nat0::binprot_read(r)?;
+        let mut buf: Vec<u8> = vec![0u8; len.0 as usize];
+        r.read_exact(&mut buf)?;
+        Ok(BufferWithLen(buf))
+    }
+}
+
+impl BinProtWrite for BufferWithLen {
+    fn binprot_write<W: std::io::Write>(&self, w: &mut W) -> Result<(), std::io::Error> {
+        let nat0 = Nat0(self.0.len() as u64);
+        nat0.binprot_write(w)?;
+        w.write_all(&self.0)?;
+        Ok(())
+    }
+}
